<?php

class OwcBase extends Model {
    
  const VERSION             = 'v2.0';  
  const VERSION_DATE        = '13-07-2018';    
  
  const UPDATE_MESSAGE_TIME = 24; // 24 hours
  
  const AJAX_MAX_LINE_LEN   = 200;  
  const AJAX_CHARS_BEFORE   = 20;  
  const AJAX_MAX_RESULTS    = 50;  
  
  const SHOP_URL            = 'https://shop.openwebcreations.eu';  
  const SUPPORT_URL         = 'https://support.openwebcreations.eu';
  const FEEDBACK_URL        = 'https://feedback.openwebcreations.eu';
 
  var $logger               = null;
  var $logfile              = 'owc.log';	  
  var $debug                = false;	  
  
  public function __construct($arg) {
	  parent::__construct($arg);  	  
  }
    		
	/**
	* get footer info
  *	 
	* @since   1.0
	*/	
	public function getOWCFooter() {
	  $footer = "<a href='". self::SHOP_URL . "' target='blank'>Open Web Creations</a> &copy; 2009-" . date("Y") . " " . $this->language->get('entry_all_rights_reserved');
	  $footer .= " - <a href='". self::SUPPORT_URL . "' target='blank'>". $this->language->get('entry_support') . "</a>";
	  $footer .= " - <a href='". self::FEEDBACK_URL . "' target='blank'>". $this->language->get('entry_feedback') . "</a>";
	  $footer .= "<br />";	  
	  return $footer;
	}				
	
	/**
	* Get updates.
	*
	* @param   string Extension name 
	*
	* @return  array with update strings 	
  *	 
	* @since   1.0
	*/	
	public function getUpdates($extension) { 
	  $updates        = array();   
	  $updateManifest = $this->OwcLoader->OwcUpdateHelper->getUpdateManifest($extension);
	  if ($updateManifest) {
      $versions = $updateManifest->xpath('//version'); 
      foreach ($versions as $version) {
        $currentVersion = (string)$this->config->get($version['name'] . '_version');	    
        $name           = $this->language->get(str_replace($extension, 'entry_ext', (string)$version['name']));         
        if (version_compare((string)$version['version'], $currentVersion, '>')) {    
          if (class_exists(str_replace('_', '', (string)$version['name']))) {        
            $updates[]  = $name . " " . $this->language->get('text_ext_name') . ";" . $version['version'] . ";".$version['name'];	  
          }
        }       
      }       
    }    
    return $updates;
  } 

  /**
	* Check if an update check must be done.
	*
	* @param   string Extension settings name for last check updates 
  *	 
	* @since   1.0
	*/	
	public function checkForUpdate($ext_check_updates, $ext_last_check_updates) {	
	  $update      = false;
	  if ($this->config->get($ext_check_updates)) {
  	  $update = true;
  	  $last_update = $this->config->get($ext_last_check_updates);  	 
  	  if (isset($last_update) && !empty($last_update)) {
  	    $start_date   = new DateTime($last_update);  	   
        $since_start  = $start_date->diff(new DateTime());       
        if (($since_start->days*24) < self::UPDATE_MESSAGE_TIME) {
          $update = false;
        }    
      } 
    }
    return $update;	  
	}
	
  /**
	* Set the time of the last update check.
  *	 
  * @param   string Extension settings name for updates 
  * @param   string Extension settings name for check updates 
  *		
	* @since   1.0
	*/
	public function updateLastCheckForUpdates($updates, $ext_updates, $ext_check_updates) {
	  if ((VERSION == '2.0.0.0')){
			$code_column = 'group';
		}else {
			$code_column = 'code';
		}
		
		$date = new DateTime('Now');
		
		$this->db->query("UPDATE `" . DB_PREFIX . "setting` SET `value` ='" . $date->format("Y-m-d H:i:s") ."' WHERE `key` = '" . $ext_check_updates. "'");   
		$this->db->query("UPDATE `" . DB_PREFIX . "setting` SET `value` ='" . $updates ."' WHERE `key` = '" . $ext_updates . "'");   
	}	
	
	/**
	* Reset the time of the last update check to force the update.
  *	 
  * @param   string Extension settings name for updates 
  * @param   string Extension settings name for check updates 
  *		
	* @since   1.0
	*/
	public function forceUpdateLastCheckForUpdates($ext_updates, $ext_check_updates) { 
	  $this->db->query("UPDATE `" . DB_PREFIX . "setting` SET `value` ='' WHERE `key` = '" . $ext_check_updates. "'");   
		$this->db->query("UPDATE `" . DB_PREFIX . "setting` SET `value` ='' WHERE `key` = '" . $ext_updates . "'");   
	}
	
	/**
	* Get path to admin module.
  *	 
	* @since   1.2
	*/
	public function getModulePath() {
	  if (version_compare(VERSION, '2.2.0.0', '<=')) {  
	    return "module/"; 
	  }

	  return "extension/module/";	
	}
	
	/**
	* Get path to admin extension module.
  *	 
	* @since   1.3
	*/
	public function getExtensionPath() {
	  if (version_compare(VERSION, '2.2.0.0', '<=')) {  
	    return "extension/module"; 
	  }
	  if (version_compare(VERSION, '2.3.0.3', '<=')) {  
	    return "extension/extension"; 
	  }
	  return "marketplace/extension";	
	}
	
	/**
	* Get path to catalog extension total.
  *	 
	* @since   1.6
	*/
	public function getExtensionTotalPath() {
	  if (version_compare(VERSION, '2.2.0.0', '<=')) {  
	    return "total/"; 
	  }
	  return "extension/total/"; 	  
	}
	
	/**
	* Get template filename.
  *	 
	* @since   1.5
	*/	
	public function getTemplate($string) {
	  if (version_compare(VERSION, '2.2.0.0', '<=')) {  
	    return $string; 
	  }
	  if (version_compare(VERSION, '2.3.0.3', '<=')) {  
	    return $string; 
	  }	  
	  //$this->config->set('template_engine', 'template'); 
	  return str_replace('.tpl', '', $string);	
	}	

	
	/**
	* Load template .tpl. Also load the old template within OpenCart 3. 
  *	 
	* @since   1.5
	*/	
	public function loadView($template, $data) {	  
	  if ($this->OwcLoader->OwcHelper->isAdminUrl()) {	
	    if (version_compare(VERSION, '2.2.0.0', '<=')) {     
	      $output = $this->load->view($this->OwcLoader->OwcBase->getModulePath() . $this->getTemplate($template), $data);	
	      return $output; 
	    } else {
	      $engine = $this->config->get('template_engine');  	 
    	  if (strpos($template, '.tpl') !== false) {    	     
    			$this->config->set('template_engine', 'template');  			
    		}	
    	  
    	  $output = $this->load->view($this->getModulePath() . $this->getTemplate($template), $data);  
    	  $this->config->set('template_engine', $engine);
    	  
    	  return $output; 
	    }
	  } else {
  	  if (version_compare(VERSION, '2.2.0.0', '<=')) {    		
    		if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/' . $this->getModulePath() . $template)) {
    			return $this->load->view($this->config->get('config_template') . '/template/' . $this->getModulePath() . $this->getTemplate($template), $data);
    		} else {    		  
    			return $this->load->view('default/template/' . $this->OwcLoader->OwcBase->getModulePath() . $this->getTemplate($template), $data);
    		}
    	} else {   		
    	  $engine     = $this->config->get('template_engine'); 
    	  $directory  = $this->config->get('template_directory');  	 
    	  if (strpos($template, '.tpl') !== false) {    	     
    			$this->config->set('template_engine', 'template');  			
    		}		 		
    		
    		//echo $this->config->get('template_engine');  		
    	  $this->config->set('template_directory', 'default/template/');
    	  
    	  $output = $this->load->view($this->getModulePath() . $this->getTemplate($template), $data);  
    	  $this->config->set('template_engine', $engine);
    	  $this->config->set('template_directory', $directory);
    	  /*$route = $this->getModulePath() . $this->getTemplate($template);  	  
    	  
    	  // Sanitize the call
    		$route = preg_replace('/[^a-zA-Z0-9_\/]/', '', (string)$route);
    		
    		// Keep the original trigger
    		$trigger = $route;
    		
    		// Template contents. Not the output!
    		$template = '';
    		
    		// Trigger the pre events
    		$result = $this->registry->get('event')->trigger('view/' . $trigger . '/before', array(&$route, &$data, &$template));
    						
    		// Make sure its only the last event that returns an output if required.
    		if ($result && !$result instanceof Exception) {
    			$output = $result;
    		} else {
    			$template = new Template('template'); //, $this->registry); //d_twig_manager.xml
    				
    			foreach ($data as $key => $value) {
    				$template->set($key, $value);
    			}
    
    			$output = $template->render($route, $this->registry->get('config')->get('template_cache'));		
    		}		
    		
    		// Trigger the post events
    		$result = $this->registry->get('event')->trigger('view/' . $trigger . '/after', array(&$route, &$data, &$output));
    		
    		if ($result && !$result instanceof Exception) {
    			$output = $result;
    		}	*/  	    	  
    	  
    	  return $output; 	  
  	  }  	
  	}
    return "";  
	}
	
	/**
	* Get user token of user token string
  *	 
	* @since   1.4
	*/
	public function getUserToken($string=true) {
	  if (version_compare(VERSION, '2.3.0.3', '<=')) {  
	    return $string ? 'token=' . $this->session->data['token'] : $this->session->data['token']; 
	  }
	  return $string ? 'user_token=' . $this->session->data['user_token'] : $this->session->data['user_token'];	
	}
	
	/**
	* addModule
  *	 
	* @since   1.4
	*/
	public function addModule($code, $data) {
	  if (version_compare(VERSION, '2.3.0.3', '<=')) {  
	    $this->load->model('extension/module');				  
	    $this->model_extension_module->addModule($code, $data);
	  } else {
	    $this->load->model('setting/module');
			$this->model_setting_module->addModule($code, $data);
	  }
	}
	
	/**
	* editModule
  *	 
	* @since   1.4
	*/
	public function editModule($module_id, $data) {
		if (version_compare(VERSION, '2.3.0.3', '<=')) {  
		  $this->load->model('extension/module');		
		  $this->model_extension_module->editModule($module_id, $data);
		} else {
	    $this->load->model('setting/module');
			$this->model_setting_module->editModule($module_id, $data);
	  }
	}
	
	/**
	* getModule
  *	 
	* @since   1.4
	*/	
	public function getModule($module_id) {
		if (version_compare(VERSION, '2.3.0.3', '<=')) {  
		  $this->load->model('extension/module');		
		  return $this->model_extension_module->getModule($module_id);
		} else {
	    $this->load->model('setting/module');
			return $this->model_setting_module->getModule($module_id);
	  }
	}
	
	/**
	* Get url_alias / seo_url table
  *	 
	* @since   1.7
	*/
	public function getUrlAliasTableName($string=true) {
	  if (version_compare(VERSION, '2.3.0.3', '<=')) {  
	    return 'url_alias'; 
	  }
	  return 'seo_url';	
	}
	
	/**
	* add Ckeditor support
  *	 
	* @since   1.9
	*/	
	public function addCkeditorSupport(&$data, $summernote=false) {
	  /* CKE editor */   
		$data['ckeditorplus_language'] = 'en';
		$data['ckeditorplus_skin'] = 'moono';
		$data['ckeditorplus_status'] = 0;		
		
		if ($this->config->get('ckeditorplus_status')) {			
				$data['ckeditorplus_status'] = $this->config->get('ckeditorplus_status');
				if ($this->config->get('ckeditorplus_language')) {$data['ckeditorplus_language'] = $this->config->get('ckeditorplus_language');}
				if ($this->config->get('ckeditorplus_skin')) {$data['ckeditorplus_skin'] = $this->config->get('ckeditorplus_skin');}				
		} else
		if (file_exists(DIR_APPLICATION . '/view/javascript/ckeditor/ckeditor.js')) {
			$data['ckeditorplus_status'] = 1;
			$this->document->addScript('view/javascript/ckeditor/ckeditor.js');
		}
		else {      
			if (version_compare(VERSION, '2.2.0.0', '>') || $summernote) {  
				$this->document->addScript('view/javascript/summernote/summernote.js');
				$this->document->addStyle('view/javascript/summernote/summernote.css');
				$this->document->addScript('view/javascript/summernote/opencart.js');
			}
		}	  
	}
	
	/**
	* Clear Nitro product cache
  *	 
	* @since   2.0
	*/
	public function clearNitroProductCache($product_id) {		   
	  if (is_dir(DIR_SYSTEM . '/nitro')) { 
      $this->load->model('tool/nitro');    
      if (getNitroPersistence('PageCache.ClearCacheOnProductEdit')) {       
        if ($product > 0) { 
          $this->model_tool_nitro->clearProductCache((int)$product_id);
        } else {
          truncateNitroProductCache();
          $filename = getQuickCacheRefreshFilename();
          touch($filename);
        }        
      }
    }
  }
	
	/**
	* Set logfile
  *	 
	* @since   1.0
	*/
	public function setLogfile($logfile) {
	  $this->logfile = $logfile;  
	}
	
	/**
	* Set log debug
  *	 
	* @since   1.0
	*/
	public function setLogDebug($debug) {
	  $this->debug = $debug;  
	}
	
	/**
	* Log info message
  *	 
	* @since   1.0
	*/
	public function logInfo($text) {
	  
	  if ($this->debug) {
	    if (!isset($this->logger)) {
	      $this->logger = new Log($this->logfile);   
	    }
	    $this->logger->write("INFO:  " . (is_array($text) ? print_r($text, true) : $text));	
	  }	
	}
	
	/**
	* Log debug message
  *	 
	* @since   1.0
	*/
	public function logDebug($text) {
	 
	  if ($this->debug) {
	    if (!isset($this->logger)) {
	      $this->logger = new Log($this->logfile);   
	    }
	    $this->logger->write("DEBUG: " . (is_array($text) ? print_r($text, true) : $text));	
	  }	
	}
	
	/**
	* Log error message
  *	 
	* @since   1.0
	*/
	public function logError($text) {	  
	  if (!isset($this->logger)) {
	    $this->logger = new Log($this->logfile);   
	  }
	  $this->logger->write("ERROR: " . (is_array($text) ? print_r($text, true) : $text));		
	}	 	
}

?>
