<?php

class OwcEUVat extends OwcBase {
  
  const VERSION                     = 'v2.9.1';  
  const VERSION_DATE                = '27-07-2018';   
  
  const SETTING_UPDATES             = 'owc_euvat_updates';
  const SETTING_CHECK_UPDATES       = 'owc_euvat_check_updates';
  const SETTING_LAST_CHECK_UPDATES  = 'owc_euvat_last_check_updates';
  
  const WSDL                        = "http://ec.europa.eu/taxation_customs/vies/checkVatService.wsdl";
  const GEO_HOST                    = "http://www.geoplugin.net/php.gp?ip={IP}&base_currency={CURRENCY}";  
  const GEO_CURRENCY                = "EUR";	
  const LOGFILE_IP_CONFIRMED        = "owc_euvat_ip_address_confirmed.log";
  
  private $soapClient       = null;  	
	
	private $iso_code_2_data = array(
		'AT' => 'AT', //Austria
		'BE' => 'BE', //Belgium
		'BG' => 'BG', //Bulgaria
		'DK' => 'DK', //Denmark
		'FI' => 'FI', //Finland
		'FR' => 'FR', //France / France mtropolitaine	OpenCart
		'FX' => 'FR', //France mtropolitaine 		
		'DE' => 'DE', //Germany
		'GR' => 'EL', //Greece OpenCart		
		'EL' => 'EL', //Greece		
		'IE' => 'IE', //Irland
		'IT' => 'IT', //Italy
		'LU' => 'LU', //Luxembourg
		'NL' => 'NL', //Netherlands
		'PT' => 'PT', //Portugal
		'ES' => 'ES', //Spain
		'SE' => 'SE', //Sweden
		'GB' => 'GB', //United Kingdom
		'CY' => 'CY', //Cyprus
		'EE' => 'EE', //Estonia
		'HU' => 'HU', //Hungary
		'LV' => 'LV', //Latvia
		'LT' => 'LT', //Lithuania
		'MT' => 'MT', //Malta
		'PL' => 'PL', //Poland
		'RO' => 'RO', //Romania
		'SK' => 'SK', //Slovakia
		'CZ' => 'CZ', //Czech Republic
		'SI' => 'SI'  //Slovania
	);
  
  public function __construct($arg) {
	  parent::__construct($arg);
	  
	  $this->setLogfile('owc_euvat.log');
	  $this->setLogdebug($this->config->get('owc_euvat_debug'));    	  
	  
	  if(!class_exists('SoapClient')) {
			if($this->debug) {
		    $this->logDebug($this->language->get('error_soapclient'));
		  }
		}		
  }
  
  /**
  * Check Soapclient available
  *
  * return Boolean true if soapclient is available.
  */
  public function checkSoapClient() {
    if(!class_exists('SoapClient')) {
      if($this->debug) {
		    $this->logDebug($this->language->get('error_soapclient'));
		  }	
		  return false;
		}		
		return true;
  }
  
  /**
  * Check IP address billing country
  *
  * return String of the error message or empty if validation is valid. 
  */
  public function checkBillingAddressAgainstIPAddress($country_id) {
    $msg = "";  
    
    // Check billing address against the IP address country.    
    $this->load->model('localisation/country');
    $country_info = $this->model_localisation_country->getCountry($country_id);
   
    if ($country_info) {
      $iso_code_2 = $country_info['iso_code_2'];
      // Check if country in EU, outside EU no IP check necessary.
      if ($this->isEUCountry($iso_code_2)) {     
	      if (!isset($this->request->post['owc_euvat_confirm_ip'])) {
	        if (!isset($this->session->data['owc_euvat_confirm_ip']) || isset($this->session->data['owc_euvat_confirm_ip_iso_code_2']) && $this->session->data['owc_euvat_confirm_ip_iso_code_2'] != $iso_code_2) {
    	      $msg      = $this->checkIPBillingCountry($iso_code_2);     
    	      if (!empty($msg)) {    
    	        if (!$this->config->get('d_quickcheckout_status') && !$this->config->get('quickcheckout_status') && 
    	            !property_exists('Front', 'IS_OC2') && $this->config->get('config_template') != 'journal2') {
    	          $msg .= sprintf($this->language->get('error_fat_number_geo_confirm_ip'), $country_info['name']);        	          	 
    	        } else {
    	          $msg .= sprintf($this->language->get('error_fat_number_geo_confirm_ip_d'), $country_info['name']);        	
    	        }      
    	      } else {
    	        unset($this->session->data['owc_euvat_confirm_ip']);    	   
  	          unset($this->session->data['owc_euvat_confirm_ip_iso_code_2']);	   
    	      }          
    	    }
  	    } else {  	         
  	      $this->session->data['owc_euvat_confirm_ip']            = 1;	      	   
  	      $this->session->data['owc_euvat_confirm_ip_iso_code_2'] = $iso_code_2;	    	       	        
  	    }      	     		     
  	  }
    }
    return $msg;   	 
  }
  
  /**
  * Check VAT number with country_id
  *
  * return String of the error message or empty if validation is valid. 
  */
  function checkEUVatCountry($country_id, $vatName, $vatNumber, $setReverse=1) {
    $msg = "";          
    
    // Check if country in EU, outside EU no VAT check necessary.
    if ($setReverse) {
      unset($this->session->data['owc_euvat_reverse_charge_vat_exempt']);	
    }      
    if (!empty($vatNumber)) {
      $country = $this->getEUVatCountry($vatNumber);
      
      if ($this->isEUCountry($country)) {           
        $msg        = $this->checkEUVat($vatName, $vatNumber, $country);   	 	     
	      if (empty($msg)) {    
	        
	        $country_id_vatNumber = $this->getCountryID($country);
	        $this->logDebug("Vatnumber country is " . $country . " / " . $country_id_vatNumber);  
	       	       
	        /*if (!$this->config->get('owc_euvat_autofill_company_address') && 
	          $country_id != $country_id_vatNumber) {   
	          // Check if VAT number can be validated, country must be filled.
            $this->load->model('localisation/country');
            $country_info = $this->model_localisation_country->getCountry($country_id);
            if ($country_info) {
              $iso_code_2 = $country_info['iso_code_2'];
	            $msg = sprintf($this->language->get('error_fat_number_invalid_country'), $country_info['name']);
  	        }
  	      } else*/  	      
  	      if ($this->config->get('config_country_id') != $country_id_vatNumber || ($this->config->get('owc_euvat_reverse_charge_vat_in_store_country') && $this->config->get('config_country_id') == $country_id_vatNumber)) {   
  	        if ($setReverse) {    	  
  	          $this->logDebug("Reverse VAT set!");         
  	          $this->session->data['owc_euvat_reverse_charge_vat_exempt'] = 1;	   
  	        }
  	      } else {
  	        $this->logDebug("Reverse VAT not in Store country!"); 
  	      }
  	    }        	   
  	  } else {
        $this->logDebug("Country with id " . $country_id . " is not an EU country!");      
        
        $msg = sprintf($this->language->get('error_fat_number_invalid_eu'), $vatName);
      }
	  }
     
    return $msg;       
  }
  
  /**
  * Check VAT number
  *
  * return String of the error message or empty if validation is valid. 
  */
  public function checkEUVat($custom_field_name, &$vatNumber, $iso_code_2) {
    $msg = "";  
                            
    // Offline VAT number check
    if (($this->config->get('owc_euvat_number_check') == 0 || $this->config->get('owc_euvat_number_check') == 2) && !($this->isValidEUVatNumber($vatNumber))) {
      $msg = sprintf($this->language->get('error_fat_number_invalid'), $custom_field_name);
    } else 
    if ($this->config->get('owc_euvat_number_check') > 0) {
      // Online VAT number check
      if ($this->checkVatNumberService()) {
        $result = $this->validate($iso_code_2, $vatNumber);
        if ($result == false) {
          $msg = sprintf($this->language->get('error_fat_number_invalid_eu'), $custom_field_name);
        } else 
        if ($result < 0) {
          if ($this->config->get('owc_euvat_online_check_unavailable') == 0) {
            $msg = $this->language->get('error_fat_number_vat_unavailable');  
          }         
        } 
      } else {
        if ($this->config->get('owc_euvat_online_check_unavailable') == 0) {
          $msg = $this->language->get('error_fat_number_vat_unavailable');
        } else 
        if (!($this->isValidEUVatNumber($vatNumber))) {
          $msg = sprintf($this->language->get('error_fat_number_invalid'), $custom_field_name); 
        }
      }    	        	        
    }
    		   	  
  	return $msg;
  }
  
  /**
  * Check IP address billing country
  *
  * return String of the error message or empty if validation is valid. 
  */
  public function checkIPBillingCountry($iso_code_2) {
    $msg = "";    

    if (empty($msg)) {
      // Online IP address country check
      if ($this->config->get('owc_euvat_geo_country_check')) {
        if ($this->checkGEOService()) {
          if ($this->getClientIP() == '127.0.0.1') {
		        return "";
		      }
		    
  		    if (!$this->checkIPCountry($iso_code_2)) {
  		      $msg = sprintf($this->language->get('error_fat_number_geo_mismatch'), $this->getClientIP(), $iso_code_2);
  		    }
  		  } else { 
  		    if ($this->config->get('owc_euvat_geo_check_unavailable') == 0) {
  		      $msg = $this->language->get('error_fat_number_geo_unavailable');  	    
  		    }
  		  }
  	  }
  	}  		   	  
  	return $msg;
  }
  
   /**
  * Check IP address country is EU country
  *
  * return String of the error message or empty if validation is valid. 
  */
  public function checkIPCountryIsEUCountry() {
    $msg = "";    
  
    if (empty($msg)) {
      // Online IP address country check      
      if ($this->checkGEOService()) {
        if ($this->getClientIP() == '127.0.0.1') {
		      return true;
		    }
		    
        return $this->checkIPEUCountry(); 
		  }		   
  	}  		   	    
  	return false;
  }
  
  /**
  * Check if the GEO service is available
  *
  * return TRUE if available 
  */
  private function checkGEOService() {
    		
		if($this->debug) {
		  $this->logDebug("Check if the GEO service is available at " . self::GEO_HOST);
		}	
		 
	  $handle = curl_init(self::GEO_HOST);
    curl_setopt($handle,  CURLOPT_RETURNTRANSFER, TRUE);
    
    $response = curl_exec($handle);
    $httpCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
    curl_close($handle);
    if($httpCode == 404) {        
        $this->logError("GEO Service is unavailable at " . self::GEO_HOST);
        return false;
    }      
    if($this->debug) {
		  $this->logDebug("GEO Service is available");
		}
    return true;
  }
  
  /**
  * Log collected data
  *
  * return TRUE if available 
  */
  public function logIPAdressConfirmed($data) {
    
    $logger = new Log(self::LOGFILE_IP_CONFIRMED);
    
    $this->load->language('checkout/checkout');		    
        
    $confirm_data = array();
    $confirm_data["Name"]       = $data['name'];
    $confirm_data["Order id"]   = $data['order_id'];   
   
    $logger->write("Logging IP address confirmed for ".  $this->getClientIP() . ": " .(is_array($confirm_data) ? print_r($confirm_data, true) : $confirm_data));	      
  }    
  
  private function checkIPCountry($country, $ip = null) {  
		
		global $_SERVER;
		
		if ( is_null( $ip ) ) {
			$ip = $this->getClientIP();
		}
		
		if($this->debug) {
		  $this->logDebug("Check country code " . $country . " for ip " . $ip);	
	  }
	  
		$host = str_replace( '{IP}', $ip, self::GEO_HOST );
		$host = str_replace( '{CURRENCY}', self::GEO_CURRENCY, $host );
		
		if($this->debug) {
		  $this->logDebug("Host url " . $host);	
	  }
	  
		$data = array();
		
		if ( function_exists('curl_init') ) {
						
			//use cURL to fetch data
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $host);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_USERAGENT, 'geoPlugin PHP Class v1.0');
			$response = curl_exec($ch);
			curl_close ($ch);
			
		} else if ( ini_get('allow_url_fopen') ) {
			
			//fall back to fopen()
			$response = file_get_contents(self::GEO_HOST, 'r');			
		} else {
		  $this->logError("GEO service cannot retrieve data. Either compile PHP with cURL support or enable allow_url_fopen in php.ini");
		}	
			
		$data = unserialize($response);		
		
		/*$this->ip = $ip;
		$this->city = $data['geoplugin_city'];
		$this->region = $data['geoplugin_region'];
		$this->areaCode = $data['geoplugin_areaCode'];
		$this->dmaCode = $data['geoplugin_dmaCode'];
		$this->countryCode = $data['geoplugin_countryCode'];
		$this->countryName = $data['geoplugin_countryName'];
		$this->continentCode = $data['geoplugin_continentCode'];
		$this->latitude = $data['geoplugin_latitude'];
		$this->longitude = $data['geoplugin_longitude'];
		$this->currencyCode = $data['geoplugin_currencyCode'];
		$this->currencySymbol = $data['geoplugin_currencySymbol'];
		$this->currencyConverter = $data['geoplugin_currencyConverter'];*/
		
		if($this->debug) {
		    $this->logDebug("Response: " . print_r($data, true));
		    $this->logDebug("Country code is " . $data['geoplugin_countryCode']);
		}
		
		if (strtolower($country) == strtolower($data['geoplugin_countryCode'])) {
		  $this->logInfo("Country code is valid!");
		  return true;
		}
		$this->logError("Country code is invalid!");
		return false;		
	}
	
	private function checkIPEUCountry($ip = null) {  
		
		global $_SERVER;
		
		if ( is_null( $ip ) ) {
			$ip = $this->getClientIP();
		}
		
		if($this->debug) {
		  $this->logDebug("Check country for ip " . $ip);	
	  }
	  
		$host = str_replace( '{IP}', $ip, self::GEO_HOST );
		$host = str_replace( '{CURRENCY}', self::GEO_CURRENCY, $host );
		
		if($this->debug) {
		  $this->logDebug("Host url " . $host);	
	  }
	  
		$data = array();
		
		if ( function_exists('curl_init') ) {
						
			//use cURL to fetch data
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $host);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_USERAGENT, 'geoPlugin PHP Class v1.0');
			$response = curl_exec($ch);
			curl_close ($ch);
			
		} else if ( ini_get('allow_url_fopen') ) {
			
			//fall back to fopen()
			$response = file_get_contents(self::GEO_HOST, 'r');			
		} else {
		  $this->logError("GEO service cannot retrieve data. Either compile PHP with cURL support or enable allow_url_fopen in php.ini");
		}	
			
		$data = unserialize($response);		
		
		/*$this->ip = $ip;
		$this->city = $data['geoplugin_city'];
		$this->region = $data['geoplugin_region'];
		$this->areaCode = $data['geoplugin_areaCode'];
		$this->dmaCode = $data['geoplugin_dmaCode'];
		$this->countryCode = $data['geoplugin_countryCode'];
		$this->countryName = $data['geoplugin_countryName'];
		$this->continentCode = $data['geoplugin_continentCode'];
		$this->latitude = $data['geoplugin_latitude'];
		$this->longitude = $data['geoplugin_longitude'];
		$this->currencyCode = $data['geoplugin_currencyCode'];
		$this->currencySymbol = $data['geoplugin_currencySymbol'];
		$this->currencyConverter = $data['geoplugin_currencyConverter'];*/
		
		if($this->debug) {
		    $this->logDebug("Response: " . print_r($data, true));
		    $this->logDebug("Country code is " . $data['geoplugin_countryCode']);
		}			
		
		if ("eu" == strtolower($data['geoplugin_continentCode']) || $this->isEUCountry(strtolower($data['geoplugin_countryCode']))) {
		  $this->logInfo("Continent code is EU!");
		  return true;
		}

		return false;		
	}
  
  /**
  * Check if the VIES VAT number service is available
  *
  * return TRUE if available 
  */
  private function checkVatNumberService() {
    		
		if($this->debug) {
		  $this->logDebug("Check if the VIES VAT number service is available at " . self::WSDL);
		}			  
	  
	  $handle = curl_init(self::WSDL);
    curl_setopt($handle,  CURLOPT_RETURNTRANSFER, TRUE);
    
    $response = curl_exec($handle);
    $httpCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
    curl_close($handle);
    if($httpCode == 404) {        
        $this->logError("SOAP Service is unavailable at " . self::WSDL);
        return false;
    }      
    if($this->debug) {
		  $this->logDebug("SOAP Service is available");
		}
    return true;
  }
  
  /**
  * Online check VAT number against the VIES VAT Number Database.
  *
  * return TRUE if VAT number is valid. 
  */
  private function validate($countryCode, $vatNumber) {    
				
		try {    
		   
			if (array_key_exists($countryCode, $this->iso_code_2_data)) {
		    $countryCode = $this->iso_code_2_data[$countryCode];   
		  }
		  
			if($this->debug) {
			  $this->logDebug("Online validate country " . $countryCode . " with VAT number " . $vatNumber);
		  }		  
		  
		  $vatNumber = strtoupper($vatNumber);
      $vatNumber = preg_replace('/[ -,.]/', '', $vatNumber);
		  
		  $vatNumber = str_replace($countryCode, "", trim($vatNumber));
		  
		  // First check if the service is available
		  $handle = curl_init(self::WSDL);
      curl_setopt($handle,  CURLOPT_RETURNTRANSFER, TRUE);
      
      $response = curl_exec($handle);
      $httpCode = curl_getinfo($handle, CURLINFO_HTTP_CODE);
      curl_close($handle);
      if($httpCode == 404) {
          /* You don't have a WSDL Service is down. exit the function */
          $this->logError("SOAP Service is unavailable at " . self::WSDL);
          return false;
      }      
			
			$this->soapClient = new SoapClient(self::WSDL, 
			  array('trace' => true, 
			  'features'=>SOAP_WAIT_ONE_WAY_CALLS,
			  'stream_context' => stream_context_create(array(
          'ssl' => array(
          'verify_peer' => false,
          'verify_peer_name' => false,
          'allow_self_signed' => true)))));
			
			$vies = $this->soapClient->checkVat( array('countryCode' => $countryCode, 'vatNumber' => $vatNumber) );
			
    	if($this->debug) {
    	  $this->logDebug('Web Service result', $this->soapClient->__getLastRequest());	    	  	  
    	}
    	
    	if($vies->valid) {   
    	  if($this->debug) {
    	    $this->logDebug("Response is valid");
    	  }
    	  
    	  $denomination = "";
    		$name         = "";
    		$vies_name = explode(" " ,$vies->name,2);
    		if (!empty($vies_name)) {
    		  if (count($vies_name) == 2) {
    		    $denomination = $vies_name[0];
    		    $name         = $vies_name[1];
    		  } else {    		  
    		    $name         = $vies_name[0];
    		  }
    		}
    	 
    		$soapRspData = array(
    								'denomination'  => 	$denomination, 
    								'name'          => 	$this->cleanUpString($denomination) . " ". $this->cleanUpString($name), 
    								'address'       => 	$this->cleanUpString($vies->address),
    								'countryCode'   => 	strtoupper($this->cleanUpString($vies->countryCode)),
    		);
    		
    		if($this->debug) {
    		  $this->logDebug($soapRspData);
    		}
    		
    		$this->session->data['owc_euvat_soap_data'] = $this->processSoapResponse($soapRspData);
    		
    		return true;
    	} else {    	
    	  if($this->debug) {
    	    $this->logDebug("Response is invalid");
    	  }
    	  
    	  unset($this->session->data['owc_euvat_soap_data']);
    		
    	  return false;
    	}
		
	  } catch(SoapFault $soape){
	    $faults = array (
        'INVALID_INPUT'       => 'The provided CountryCode is invalid or the VAT number is empty',
        'SERVICE_UNAVAILABLE' => 'The SOAP service is unavailable, try again later',
        'MS_UNAVAILABLE'      => 'The Member State service is unavailable, try again later or with another Member State',
        'TIMEOUT'             => 'The Member State service could not be reached in time, try again later or with another Member State',
        'SERVER_BUSY'         => 'The service cannot process your request. Try again later.'
      );
      $error = $faults[$soape->faultstring];
        
      // handle issues returned by the web service
      $this->logError("SOAP Fault: (faultcode: {$soape->faultcode}, faultstring: {$soape->faultstring}, error: " . $error . ")", E_USER_ERROR);
      unset($this->session->data['owc_euvat_soap_data']);      
      return -1;
    } catch(Exception $e) {
			$this->logError('Vat Translation Error', $e->getMessage());
			unset($this->session->data['owc_euvat_soap_data']);			
			return -1;
		}
	}	  
	
	private function processSoapResponse ($soapRspData) {		
		$address = $this->decodeAddress($soapRspData['address'], $soapRspData['countryCode']);
		$country =  $this->getCountryID($soapRspData['countryCode']);
		if ($soapRspData['countryCode'] == 'DE' || $soapRspData['countryCode'] == 'ES') {
			$soapRspData['name'] = '';
		}
		
		$result = array(
		  'country'	  => !empty($country) ? ucfirst($country) : '', 
			'company'		=> $soapRspData['name'],
			'address-1' => !empty($address['address_1']) ? ucfirst($address['address_1']) : '' ,
			'address-2' => !empty($address['address_2']) ? ucfirst($address['address_2']) : '' ,
			'postcode' 	=> !empty($address['pc']) ? strtoupper($address['pc']) : '' ,
			'city'		  => !empty($address['city']) ? ucfirst($address['city']) : ''			
		);
		
		$result = array_map('trim', $result);		
		
		if($this->debug) {
	    $this->logDebug("SOAP Address: " . print_r($result, true));
	  }
		
		return $result;
	}
	
	private function decodeAddress($line, $country) {
		$address = array();
		
		switch ($country) {
			case 'BE':
				$exploded             = explode("\n",$line);
				$pc_city              = explode(' ', isset($exploded[1]) ? $exploded[1] : '');
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['pc']        = isset($pc_city[0]) ? $pc_city[0] : '';
				$address['city']      = isset($pc_city[1]) ? $pc_city[1] : '';
				$this->splitAddressField($address);
				break;
			case 'BG':
				$exploded = explode(",",$line);
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['address_2'] = isset($exploded[1]) ? $exploded[1] : '';
				break;
			case 'DE':
			case 'ES':
				break;
			case 'EE':
				$address['address_1'] = $line;
				$this->splitAddressField($address);
				break;
			case 'EL':
				$exploded             = explode(" 0 ",$line);
				$pc_city              = explode(' - ', isset($exploded[1]) ? $exploded[1] : '');
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['pc']        = isset($pc_city[0]) ? $pc_city[0] : '';
				$address['city']      = isset($pc_city[1]) ? $pc_city[1] : '';
				$this->splitAddressField($address);
				break;
			case 'NL':
				$exploded             = explode("\n",$line);
				$pc_city              = explode(' ', isset($exploded[1]) ? $exploded[1] : '');
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['pc']        = isset($pc_city[0]) ? $pc_city[0] : '';
				$address['city']      = isset($pc_city[1]) ? $pc_city[1] : '';
				$this->splitAddressField($address);
				break;
			case 'MT':
				$exploded = explode("\n",$line);
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['address_2'] = $exploded[1] . ', ' . $exploded[2];
				$address['city']      = array_pop($exploded);
				$address['pc']        = end($exploded);
				
				break;
			case 'GB':
				$exploded = explode("\n",$line);
				
				$address['address_1'] = isset($exploded[1]) ? $exploded[1] : '';
				$address['address_2'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['pc']        = isset($exploded[5]) ? $exploded[5] : '';
				$address['city']      = isset($exploded[3]) ? $exploded[3] : '';
				break;
			case 'PT':
				$exploded             = explode("\n",$line);
				$pc_city              = explode(' ', end($exploded));
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['address_2'] = isset($exploded[1]) ? $exploded[1] : '';
				if(!empty($pc_city)) {
					$address['pc']    = !empty($pc_city[0]) ? array_shift($pc_city) : '';
					$address['city']  = !empty($pc_city) ? str_replace($address['pc'], '' , end($exploded)) : '';
				}
				break;
			case 'CZ':
				$exploded             = explode("\n",$line);
				$pc_city              = explode(' ', end($exploded));
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['address_2'] = isset($exploded[1]) ? $exploded[1] : '';
				if(!empty($pc_city)) {
					$address['pc']    = !empty($pc_city[0]) ? $pc_city[0] : '';
					$address['city']  = !empty($pc_city[1]) ? $pc_city[1] : '';
				}
				break;
			case 'IE':
				$exploded             = explode(",",$line);
				$pc_city              = explode(' ', end($exploded));
				$exploded             = array_map('trim', $exploded);
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['address_2'] = isset($exploded[1]) ? $exploded[1] : '' . isset($exploded[2]) ? ', ' . $exploded[2] : '';
				$address['pc']        = isset($pc_city[1]) ? $pc_city[1] : '';
				$address['city']      = isset($pc_city[0]) ? $pc_city[0] : '';
				break;
			case 'LU':
				$exploded             = explode("\n",$line);
				$pc_city              = explode(' ', $exploded[1]);
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['pc']        = isset($pc_city[0]) ? $pc_city[0] : ''; 
				$address['city']      = isset($pc_city[1]) ? $pc_city[1] : '';
				$this->splitAddressField($address);
				break;
			case 'RO':
				$exploded             = explode("\n",$line);
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['address_2'] = isset($exploded[1]) ? $exploded[1] : '';
				break;
			case 'SI':
				$exploded             = explode(",",$line);
				$pc_city              = explode(' ', trim(end($exploded)));
				
				$address['address_1']    = isset($exploded[0]) ? $exploded[0] : '';
				if (!empty($exploded[1])) {
					$address['address_2'] = isset($exploded[1]) ? $exploded[1] : '';
				} 
				if (!empty($exploded[2]) && $exploded[2] != end($exploded) ) {
					$address['address_2'] .= ', '.$exploded[2];
				}
				$address['pc']        = array_shift($pc_city);;
				$address['city']      = str_replace($address['pc'], '' , end($exploded));
				break;
				
			default:
				$exploded             = explode("\n",$line);
				$pc_city              = explode(' ', $exploded[1]);
				
				$address['address_1'] = isset($exploded[0]) ? $exploded[0] : '';
				$address['pc']        = array_shift($pc_city);
				$address['city']      = str_replace($address['pc'], '' , $exploded[1]);
				$this->splitAddressField($address);
				break;		
		}
			
		return $address;
	}
	
	public function splitAddressField(&$address) {	  
	  if ($this->config->get('owc_euvat_autofill_split_address')) {
		  $explodedaddress      = explode(" ",$address['address_1']); 
		  $address['address_2'] = isset($explodedaddress[0]) ? ltrim($explodedaddress[sizeof($explodedaddress)-1], '0') : '';
		  unset($explodedaddress[sizeof($explodedaddress)-1]);
		  $address['address_1'] = isset($explodedaddress[0]) ? implode(" ",$explodedaddress) : '';
	  }
	}
	
	public function getCountryID($iso_code_2){		 
		$key = array_search (strtoupper($iso_code_2), $this->iso_code_2_data, true);
		
		if ($key !== false) {		 
	    $iso_code_2 = $key;   
	  }	  	 
		
		$query = $this->db->query("SELECT country_id FROM `" . DB_PREFIX . "country` WHERE iso_code_2 = '" . $this->db->escape($iso_code_2) . "'");
		if (!empty($query->num_rows)) {
			return $query->row['country_id'];
		}		
	}
	
	/**
  * Check if supplied country code is in EU.  
  *
  * @param string $iso_code_2 Country iso 2 code, e.g. AT
  * 
  * Return TRUE if country code is an EU country code.
  */
	public function isEUCountry($iso_code_2) {
	  $iso_code_2 = strtoupper($iso_code_2);
	  if (array_key_exists($iso_code_2, $this->iso_code_2_data)) {
	    $this->logDebug("Country " . $iso_code_2. " is a valid EU country!");
	    return true;  
	  }
	  $this->logDebug("Country " . $iso_code_2. " is NOT a valid EU country!");
	  return false;
	}
	 
  
  /**
  * Return TRUE if supplied tax ID is valid for supplied country.
  *
  * @param string $vatNumber Taxation ID, e.g. ATU99999999 for Austria.
  *
  * Based on JS validator by John Gardner: http://www.braemoor.co.uk/software/vat.shtml 
  */
  private function isValidEUVatNumber(&$vatNumber){
    
    if($this->debug) {
		  $this->logDebug("Offline validate VAT number " . $vatNumber);
		}
		  
    $vatNumber = strtoupper($vatNumber);
    $vatNumber = preg_replace('/[ -,.]/', '', $vatNumber);
    if(strlen($vatNumber) < 8){
      $this->logDebug("VAT number format is not valid, small then 8 characters!");
      return false;
    }
    
    $country = substr($vatNumber, 0, 2);
    switch($country){
      case 'AT': // AUSTRIA
          $isValid = (bool) preg_match('/^(AT)U(\d{8})$/', $vatNumber);
          break;
      case 'BE': // BELGIUM
          $isValid = (bool) preg_match('/(BE)(0?\d{9})$/', $vatNumber);
          break;
      case 'BG': // BULGARIA
          $isValid = (bool) preg_match('/(BG)(\d{9,10})$/', $vatNumber);
          break;
      case 'CHE': // Switzerland
          $isValid = (bool) preg_match('/(CHE)(\d{9})(MWST)?$/', $vatNumber);
          break;
      case 'CY': // CYPRUS
          $isValid = (bool) preg_match('/^(CY)([0-5|9]\d{7}[A-Z])$/', $vatNumber);
          break;
      case 'CZ': // CZECH REPUBLIC
          $isValid = (bool) preg_match('/^(CZ)(\d{8,10})(\d{3})?$/', $vatNumber);
          break;
      case 'DE': // GERMANY
          $isValid = (bool) preg_match('/^(DE)([1-9]\d{8})/', $vatNumber);
          break;
      case 'DK': // DENMARK
          $isValid = (bool) preg_match('/^(DK)(\d{8})$/', $vatNumber);
          break;
      case 'EE': // ESTONIA
          $isValid = (bool) preg_match('/^(EE)(10\d{7})$/', $vatNumber);
          break;
      case 'EL': // GREECE
          $isValid = (bool) preg_match('/^(EL)(\d{9})$/', $vatNumber);
          break;
      case 'ES': // SPAIN
          $isValid = (bool) preg_match('/^(ES)([A-Z]\d{8})$/', $vatNumber)
              || preg_match('/^(ES)([A-H|N-S|W]\d{7}[A-J])$/', $vatNumber)
              || preg_match('/^(ES)([0-9|Y|Z]\d{7}[A-Z])$/', $vatNumber)
              || preg_match('/^(ES)([K|L|M|X]\d{7}[A-Z])$/', $vatNumber);
          break;
      case 'EU': // EU type
          $isValid = (bool) preg_match('/^(EU)(\d{9})$/', $vatNumber);
          break;
      case 'FI': // FINLAND
          $isValid = (bool) preg_match('/^(FI)(\d{8})$/', $vatNumber);
          break;
      case 'FR': // FRANCE
          $isValid = (bool) preg_match('/^(FR)(\d{11})$/', $vatNumber)
              || preg_match('/^(FR)([(A-H)|(J-N)|(P-Z)]\d{10})$/', $vatNumber)
              || preg_match('/^(FR)(\d[(A-H)|(J-N)|(P-Z)]\d{9})$/', $vatNumber)
              || preg_match('/^(FR)([(A-H)|(J-N)|(P-Z)]{2}\d{9})$/', $vatNumber);
          break;
      case 'GB': // GREAT BRITAIN
          $isValid = (bool) preg_match('/^(GB)?(\d{9})$/', $vatNumber)
              || preg_match('/^(GB)?(\d{12})$/', $vatNumber)
              || preg_match('/^(GB)?(GD\d{3})$/', $vatNumber)
              || preg_match('/^(GB)?(HA\d{3})$/', $vatNumber);
          break;
      case 'GR': // GREECE
          $isValid = (bool) preg_match('/^(GR)(\d{8,9})$/', $vatNumber);
          break;
      case 'HR': // CROATIA
          $isValid = (bool) preg_match('/^(HR)(\d{11})$/', $vatNumber);
          break;
      case 'HU': // HUNGARY
          $isValid = (bool) preg_match('/^(HU)(\d{8})$/', $vatNumber);
          break;
      case 'IE': // IRELAND
          $isValid = (bool) preg_match('/^(IE)(\d{7}[A-W])$/', $vatNumber)
              || preg_match('/^(IE)([7-9][A-Z\*\+)]\d{5}[A-W])$/', $vatNumber)
              || preg_match('/^(IE)(\d{7}[A-W][AH])$/', $vatNumber);
          break;
      case 'IT': // ITALY
          $isValid = (bool) preg_match('/^(IT)(\d{11})$/', $vatNumber);
          break;
      case 'LV': // LATVIA
          $isValid = (bool) preg_match('/^(LV)(\d{11})$/', $vatNumber);
          break;
      case 'LT': // LITHUNIA
          $isValid = (bool) preg_match('/^(LT)(\d{9}|\d{12})$/', $vatNumber);
          break;
      case 'LU': // LUXEMBOURG
          $isValid = (bool) preg_match('/^(LU)(\d{8})$/', $vatNumber);
          break;
      case 'MT': // MALTA
          $isValid = (bool) preg_match('/^(MT)([1-9]\d{7})$/', $vatNumber);
          break;
      case 'NL': // NETHERLAND
          $isValid = (bool) preg_match('/^(NL)(\d{9})B\d{2}$/', $vatNumber);
          break;
      case 'NO': // NORWAY
          $isValid = (bool) preg_match('/^(NO)(\d{9})$/', $vatNumber);
          break;
      case 'PL': // POLAND
          $isValid = (bool) preg_match('/^(PL)(\d{10})$/', $vatNumber);
          break;
      case 'PT': // PORTUGAL
          $isValid = (bool) preg_match('/^(PT)(\d{9})$/', $vatNumber);
          break;
      case 'RO': // ROMANIA
          $isValid = (bool) preg_match('/^(RO)([1-9]\d{1,9})$/', $vatNumber);
          break;
      case 'RS': // SERBIA
          $isValid = (bool) preg_match('/^(RS)(\d{9})$/', $vatNumber);
          break;
      case 'SI': // SLOVENIA
          $isValid = (bool) preg_match('/^(SI)([1-9]\d{7})$/', $vatNumber);
          break;
      case 'SK': // SLOVAK REPUBLIC
          $isValid = (bool) preg_match('/^(SK)([1-9]\d[(2-4)|(6-9)]\d{7})$/', $vatNumber);
          break;
      case 'SE': // SWEDEN
          $isValid = (bool) preg_match('/^(SE)(\d{10}01)$/', $vatNumber);
          break;
      default:
          $isValid = false;
    }
    
    if($this->debug) {
      if ($isValid) {
		    $this->logDebug("VAT number format is valid!");
		  } else {
		    $this->logDebug("VAT number format is invalid!");
		  }
		}  

    return $isValid;
  }
  
  /**
  * Get the country code of the VAT number.
  *
  * @param string $vatNumber Taxation ID, e.g. ATU99999999 for Austria.
  *  
  */
  private function getEUVatCountry($vatNumber) {
    
    $vatNumber = strtoupper($vatNumber);
    $vatNumber = preg_replace('/[ -,.]/', '', $vatNumber);
    if(strlen($vatNumber) < 8){
      $this->logDebug("VAT number format is not valid, small then 8 characters!");
      return "";
    }
    
    return substr($vatNumber, 0, 2);
  }
  
  private function formatEUVatNumber($vatNumber) {
    return preg_replace('/[ -,.]/', '', $vatNumber);
  }
  
  private function getClientIP() {
    
    // Get client's IP address
    if (isset($_SERVER['HTTP_CLIENT_IP']) && array_key_exists('HTTP_CLIENT_IP', $_SERVER)) {
        $ip = $_SERVER['HTTP_CLIENT_IP'];
    } elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && array_key_exists('HTTP_X_FORWARDED_FOR', $_SERVER)) {
        $ips = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
        $ips = array_map('trim', $ips);
        $ip = $ips[0];
    } elseif (isset($_SERVER['REMOTE_ADDR'])) {
        $ip = $_SERVER['REMOTE_ADDR'];
    } else {
        $ip = $_SERVER['REMOTE_ADDR'] = '0.0.0.0';
    }
    
    $ip = filter_var($ip, FILTER_VALIDATE_IP);
    $ip = ($ip === false) ? '0.0.0.0' : $ip;
    return $ip;
  }  
  
  /**
	* Cleanup string
  *	 
	* @since   1.0
	*/
  private function cleanUpString($string) {
    
    if ($string == '---') {
      return "";  
    }
    
    for($i=0;$i<100;$i++)
    {               
      $newString = str_replace("  "," ",$string);
      if($newString === $string) {
      	break;
      } else {
      	$string = $newString;
      }
    }
                
    $newString = "";
    $words = explode(" ",$string);
    foreach($words as $k=>$w)
    {                       
      $newString .= ucfirst(strtolower($w))." "; 
    }                
    return trim($newString);
	} 
	
	/**
	* Ajax validate
  *	 
	* @since   2.7
	*/
  function ajaxCheckVatnumber($admin=false) {	
    $this->logDebug('ajaxCheckVatnumber'); 
	  $json = array();	  
	  
	  $json['autofill_company_address'] = $this->config->get('owc_euvat_autofill_company_address');
	  $json['autofill_disable_fields']  = false;
	  $json['autofill_data']            = array('company'    => '', 'address-1'  => '', 'address-2'  => '', 
	                                            'postcode'   => '', 'city'       => '', 'country'    => '');   	 
	  
	  if (isset($this->request->post['vatnumber']) && isset($this->request->post['country_id'])) { 
	    
	    if (isset($this->request->post['customer_group_id']) && $this->request->post['customer_group_id'] != -1) {
	      $customer_group_id = $this->request->post['customer_group_id'];
	    } else {  
	      $customer_group_id = $this->config->get('config_customer_group_id');
	    }
	    
	    $this->logDebug('customer_group_id=' . $customer_group_id); 
	    $this->logDebug('vatnumber=' . $this->request->post['vatnumber']); 
	    $this->logDebug('country_id=' . $this->request->post['country_id']); 
	    // Custom Field name    
      if ($admin) {		 
        if (version_compare(VERSION, '2.0.3.1', '>')) {
          $this->language->load('customer/customer');	
          $this->load->model('customer/custom_field');
          $custom_field         = $this->model_customer_custom_field->getCustomField($this->config->get('owc_euvat_custom_field_id')); 
          $custom_fields        = $this->model_customer_custom_field->getCustomFields($customer_group_id);
        } else {
          $this->language->load('sale/customer');	
          $this->load->model('sale/custom_field');
          $custom_field         = $this->model_sale_custom_field->getCustomField($this->config->get('owc_euvat_custom_field_id'));
          $custom_fields        = $this->model_sale_custom_field->getCustomFields($customer_group_id);
        }			   
      } else {            
        $this->language->load('account/address');	
        
        $this->load->model('account/custom_field');
        $custom_field     = $this->model_account_custom_field->getCustomField($this->config->get('owc_euvat_custom_field_id'));
        $custom_fields    = $this->model_account_custom_field->getCustomFields($customer_group_id);
      }
      
      // Check custom fields in customer_group_id	  
		  $custom_field_vat = false;		  
		  foreach($custom_fields as $element) {
  		  if ($element['custom_field_id'] == $this->config->get('owc_euvat_custom_field_id')) {
  		    $custom_field_vat = true;  		    
  		    break;
  		  }
  		}  		
      
      $custom_field_name  = $custom_field['name'];  	        
      if ($custom_field_vat && !empty($this->request->post['vatnumber'])) {		      
	      
  		  $msg = $this->OwcLoader->OwcEUVat->checkEUVatCountry($this->request->post['country_id'], $custom_field_name, $this->request->post['vatnumber']);	        	  
  	   
  	    if (!empty($msg)) {        	      
  	      $json['error'] = $msg;       	      	    
  	    } 
  	  
  	    if (!isset($json['error'])) {    	      
  	      $json['autofill_disable_fields']  = $this->config->get('owc_euvat_autofill_disable_fields');
  	      	       
  	      $json['success'] = sprintf($this->language->get('success_fat_number_valid'), $custom_field_name);  
  	      
  	      if ($this->config->get('owc_euvat_format')) {
  	        $json['vatnumber'] = $this->formatEUVatNumber($this->request->post['vatnumber']);
  	      }
  	      
  	      if (isset($this->session->data['owc_euvat_soap_data']) && $this->config->get('owc_euvat_autofill_company_address')) {
  	        $json['autofill_data'] = $this->session->data['owc_euvat_soap_data'];              	  
        	} else {
        	  // Disable country in case of a valid vat number
        	  $json['autofill_company_address'] = true;
        	  $json['autofill_disable_fields']  = true;
	          $json['autofill_data']            = array('country'    => $this->getCountryID($this->getEUVatCountry($this->request->post['vatnumber'])));     
        	}
        }
  	  } else {
  	    unset($this->session->data['owc_euvat_reverse_charge_vat_exempt']);	
  	    $json['reload'] = true; 
  	    //$json['error'] = sprintf($this->language->get('error_fat_number_invalid'), $custom_field_name);     
  	  } 	       	 
  	  
  	  if ($this->config->get('d_quickcheckout_status')) {
        $json['d_quickcheckout'] = true;
      } 
      if ($this->config->get('quickcheckout_status')) {
        $json['quickcheckout'] = true;
      } 
      if ($this->config->get('onepagecheckout_status')) {
        $json['onepagecheckout'] = true;              
      }  
      if (property_exists('Front', 'IS_OC2') && $this->config->get('config_template') == 'journal2') {
        $json['journal'] = true;
      }
      
      if (!isset($this->session->data['owc_euvat_reverse_charge_vat_exempt_vatnumber']) || $this->session->data['owc_euvat_reverse_charge_vat_exempt_vatnumber'] != $this->request->post['vatnumber'] ||
          !isset($this->session->data['owc_euvat_reverse_charge_vat_exempt_country_id']) || $this->session->data['owc_euvat_reverse_charge_vat_exempt_country_id'] != $this->request->post['country_id'] ) {
        $json['reload'] = true;        
      }
      
      
      $this->session->data['owc_euvat_reverse_charge_vat_exempt_vatnumber']  = $this->request->post['vatnumber']; 
  	  $this->session->data['owc_euvat_reverse_charge_vat_exempt_country_id'] = $this->request->post['country_id'];     
    }  	    
	  
    $this->response->addHeader('Content-Type: application/json');
    $this->response->setOutput(json_encode($json));	
  } 
		
	/**
	* get footer info
  *	 
	* @since   1.0
	*/	
	public function getOWCFooter() {
	  $footer   = $this->OwcLoader->OwcBase->getOWCFooter();
	  $versions = array();
	  
	  $versions[] = $this->language->get('entry_ext') . " <strong>" . self::VERSION. "</strong>";
	  
	  $footer .= "(";
	  $footer .= implode($versions, ' - ');
	  $footer .= ")";
	    	  
	  $footer .= "<br />-<br />";
	  return '<footer id="footer" style="height: 64px">' . $footer . '</footer>';    
	}		
	
	public function install() {
	  if ((VERSION == '2.0.0.0')){
			$code_column = 'group';
		}else {
			$code_column = 'code';
		}		
		
		$installed  = $this->config->get('owc_euvat_installed');
		$version    = $this->config->get('owc_euvat_version');
		
		if (empty($installed)) {	
           
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_status', '1', '0')");           
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'module_owc_euvat', 'module_owc_euvat_status', '1', '0')");   
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_custom_field_id', '-1', '0')");     
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_reverse_charge_vat_in_store_country', '0', '0')");       	    
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_number_check', '2', '0')");  
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_online_check_unavailable', '1', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_fat_number_invalid', '0', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_geo_country_check', '1', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_format', '1', '0')");      
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_auto_check', '1', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_autofill_disable_fields', '1', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_autofill_company_address', '1', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_autofill_split_address', '1', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_geo_check_unavailable', '1', '0')");
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_debug', '0', '0')"); 
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', '".self::SETTING_CHECK_UPDATES."', '1', '0')"); 
     
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat_installer', 'owc_euvat_installed', '1', '0')");  
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat_installer', '".self::SETTING_LAST_CHECK_UPDATES."', '', '0')");    	  
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat_installer', '".self::SETTING_UPDATES."', '', '0')");    	
      $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat_installer', 'owc_euvat_version', '" . self::VERSION . "', '0')"); 
    } else {
      
      if (version_compare($version, 'v2.4', '<=')) {  	
        $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'module_owc_euvat', 'module_owc_euvat_status', '" . $this->config->get('owc_euvat_status') . "', '0')");  
        $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_fat_number_invalid', '0', '0')");  
      }
      
      if (version_compare($version, 'v2.7', '<=')) {  	
        $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_format', '1', '0')");
        $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_auto_check', '1', '0')");
        $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_autofill_disable_fields', '1', '0')");
        $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_autofill_company_address', '0', '0')");
      }
      
      if (version_compare($version, 'v2.9', '<=')) {
        $this->db->query("INSERT INTO `" . DB_PREFIX . "setting` (`setting_id`, `store_id`, `".$code_column."`, `key`, `value`, `serialized`) VALUES (NULL, '0', 'owc_euvat', 'owc_euvat_autofill_split_address', '1', '0')");
      }
      
      $this->db->query("UPDATE `" . DB_PREFIX . "setting` SET `value` = '" . self::VERSION . "' WHERE `key` = 'owc_euvat_version'"); 
    }
  }
	
	public function uninstall() {
	  if ((VERSION == '2.0.0.0')){
			$code_column = 'group';
		}else {
			$code_column = 'code';
		}
  
	  $this->db->query("DELETE FROM `" . DB_PREFIX . "setting` WHERE `".$code_column."` = 'owc_euvat' AND `key` like 'owc_euvat_%'");
	  $this->db->query("DELETE FROM `" . DB_PREFIX . "setting` WHERE `".$code_column."` = 'owc_euvat_installer'");  
	}	
}

?>
