<?php

class OwcHelper extends Model {		
	
	const VERSION             = 'v1.8';  
  const VERSION_DATE        = '17-04-2018';  
  
  /**
	 * Check if current url is a admin url. 
	 *	 
	 * @since   1.0
	 */
	function isAdminUrl() {			  
	  return (!(strpos($this->getRequestURI(), 'token') === false) || !(strpos($this->getRequestURI(), 'token') === false)) && (strpos($this->getRequestURI(), 'api') === false);
	}
	
	/**
	 * Get admin folder name. 
	 *	 
	 * @since   1.3
	 */
	function getAdminFolder() {
	  $folder=str_replace('\\','/',trim(DIR_APPLICATION));
    if (substr($folder,-1)=='/') $folder=substr($folder,0,-1);
	  $path = explode('/', $folder);
    return array_pop($path);	  
	}
	
	/**
	 * Get catalog folder name. 
	 *	 
	 * @since   1.3
	 */
	function getCatalogFolder() {
	  if (defined('DIR_CATALOG')) {
	    $folder=str_replace('\\','/',trim(DIR_CATALOG));
	  } else {
	    $folder=str_replace('\\','/',trim(DIR_APPLICATION));  
	  }	  
    if (substr($folder,-1)=='/') $folder=substr($folder,0,-1);
	  $path = explode('/', $folder);	  
    return array_pop($path);	  
	}
	
	function getTemplatePrefix($url=null)
	{
	  if (!isset($url)) {
	    $url = $_SERVER['SERVER_NAME'];
	  }
	  $url = explode('.', $url);	
	  if (count($url) > 1 )	
		  return strtolower($url[1]);	   
		return strtolower($url[0]);	     
	}			  
	
	function getRequestURI()
	{
		$pageURL = (@$_SERVER["HTTPS"] == "on") ? "https://" : "http://";
		return $pageURL . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];	
	}
	
	public function getCurrentUrl($request_uri = true) {
        if (
            isset($_SERVER['HTTPS']) && ( $_SERVER['HTTPS'] == 'on' || $_SERVER['HTTPS'] == 1 ) || isset($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https'
        ) {
            $protocol = 'https://';
        } else {
            $protocol = 'http://';
        }

        $url = $protocol . $_SERVER['HTTP_HOST'];

        if (isset($_SERVER['SERVER_PORT']) && strpos($url, ':' . $_SERVER['SERVER_PORT']) === FALSE) {
            $url .= ($protocol === 'http://' && $_SERVER['SERVER_PORT'] != 80 && !isset($_SERVER['HTTP_X_FORWARDED_PROTO'])) || ($protocol === 'https://' && $_SERVER['SERVER_PORT'] != 443 && !isset($_SERVER['HTTP_X_FORWARDED_PROTO'])) ? ':' . $_SERVER['SERVER_PORT'] : '';
        }

        if ($request_uri) {
            $url .= $_SERVER['REQUEST_URI'];
        } else {
            $url .= $_SERVER['PHP_SELF'];
        }

        // return current url
        return $url;
    }
	
	/**
	 * Format url without http and https. 
	 *	 
	 * @since   1.0
	 */
	function website_format($url, $domain_only=0)
	{
	  if ($domain_only) {
	    $url = parse_url($this->url_format($url), PHP_URL_HOST);	    
	  }
	  
		if (strlen($url) > 0) {
			if (substr($url, 0, 7) == "http://")
				$url = substr($url, 7);
			if (substr($url, 0, 8) == "https://")
				$url = substr($url, 8);
		}		
		return rtrim($url, '/');	
	}
	
	/**
	 * Format url with http or https. 
	 *	 
	 * @since   1.0
	 */
	function url_format($url, $https=1)
	{
		if (strlen($url) > 0) {
			if (substr($url, 0, 7) == "http://")
				return rtrim($url, '/');	
			if (substr($url, 0, 8) == "https://")
				return rtrim($url, '/');	
			
			$protocol = isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on' && $https ? "https://" : "http://";  
					
			return $protocol.rtrim($url, '/');		
		}
		return "";
	}
	
	/**
	 * Validate date
	 *	 
	 * @since   1.0
	 */
	public function validateDate($date, $format = "d/m/Y") {
    if (!empty($date)) {
      $dt = DateTime::createFromFormat($format, $date);
      
      if ($format == 'd/m/Y') {   
        if (!preg_match("/^[0-3][0-9]\/[0-1][0-9]\/[0-9]{4}$/",$date)) {
          return false;
        }        
      } else {
        if (!preg_match("/^[0-3][0-9]-[0-1][0-9]-[0-9]{4}$/",$date)) {
          return false;
        }
      }          
      
      //if ($dt !== false && !array_sum($dt->getLastErrors())) {
        return true;
      //}
    }
    
    return false;      
  }	
	
	/**
	 * Check if url is internal or external
	 *	 
	 * @since   1.0
	 */
	public function isInternal($url)
	{		  
    if (strpos($url, $_SERVER['HTTP_HOST']) !== false) return true;  // contains host
    
		if (!filter_var($url, FILTER_VALIDATE_URL) === false) {		  
      return false;
    }     
   
    $components = parse_url($url);
    if (empty($components['host']) && strpos($components['path'], 'www.') === false) return true;
    return !preg_match('/^http:\/\/|(www\.)?[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/', $url, $matches);   
  }	
	
 	/**
  * Truncates text.
  *
  * Cuts a string to the length of $length and replaces the last characters
  * with the ending if the text is longer than length.
  *
  * ### Options:
  *
  * - `ending` Will be used as Ending and appended to the trimmed string
  * - `exact` If false, $text will not be cut mid-word
  * - `html` If true, HTML tags would be handled correctly
  *
  * @param string  $text String to truncate.
  * @param integer $length Length of returned string, including ellipsis.
  * @param array $options An array of html attributes and options.
  * @return string Trimmed string.
  * @access public
  * @link http://book.cakephp.org/view/1469/Text#truncate-1625
  */
  function truncate($text, $length = 100, $options = array()) {
    $default = array(
        'ending' => '...', 'exact' => true, 'html' => false
    );
    $options = array_merge($default, $options);
    extract($options);

    if ($html) {
      if (strlen(preg_replace('/<.*?>/', '', $text)) <= $length) {
          return $text;
      }
      $totalLength = strlen(strip_tags($ending));
      $openTags = array();
      $truncate = '';

      preg_match_all('/(<\/?([\w+]+)[^>]*>)?([^<>]*)/', $text, $tags, PREG_SET_ORDER);
      foreach ($tags as $tag) {
          if (!preg_match('/img|br|input|hr|area|base|basefont|col|frame|isindex|link|meta|param/s', $tag[2])) {
              if (preg_match('/<[\w]+[^>]*>/s', $tag[0])) {
                  array_unshift($openTags, $tag[2]);
              } else if (preg_match('/<\/([\w]+)[^>]*>/s', $tag[0], $closeTag)) {
                  $pos = array_search($closeTag[1], $openTags);
                  if ($pos !== false) {
                      array_splice($openTags, $pos, 1);
                  }
              }
          }
          $truncate .= $tag[1];

          $contentLength = strlen(preg_replace('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', ' ', $tag[3]));
          if ($contentLength + $totalLength > $length) {
              $left = $length - $totalLength;
              $entitiesLength = 0;
              if (preg_match_all('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', $tag[3], $entities, PREG_OFFSET_CAPTURE)) {
                  foreach ($entities[0] as $entity) {
                      if ($entity[1] + 1 - $entitiesLength <= $left) {
                          $left--;
                          $entitiesLength += strlen($entity[0]);
                      } else {
                          break;
                      }
                  }
              }

              $truncate .= substr($tag[3], 0 , $left + $entitiesLength);
              break;
          } else {
              $truncate .= $tag[3];
              $totalLength += $contentLength;
          }
          if ($totalLength >= $length) {
              break;
          }
      }
    } else {
      if (strlen($text) <= $length) {
          return $text;
      } else {
          $truncate = substr($text, 0, $length - strlen($ending));
      }
    }
    if (!$exact) {
        $spacepos = strrpos($truncate, ' ');
        if (isset($spacepos)) {
            if ($html) {
                $bits = substr($truncate, $spacepos);
                preg_match_all('/<\/([a-z]+)>/', $bits, $droppedTags, PREG_SET_ORDER);
                if (!empty($droppedTags)) {
                    foreach ($droppedTags as $closingTag) {
                        if (!in_array($closingTag[1], $openTags)) {
                            array_unshift($openTags, $closingTag[1]);
                        }
                    }
                }
            }
            $truncate = substr($truncate, 0, $spacepos);
        }
    }
    $truncate .= $ending;

    if ($html) {
        foreach ($openTags as $tag) {
            $truncate .= '</'.$tag.'>';
        }
    }

    return $truncate;
  }
	
	function clearElementTag($startel, $eindel, &$text)
	{
	  $text	= str_replace($startel, "", $text);
		$text	= str_replace($eindel, "", $text);
	}
	
	function removeElementTag($startel, $eindel, &$text)
	{
		do {		
			$elstart = strpos($text, $startel);
			if (!($elstart === false)) {
				$eleind  = strpos($text, $eindel);	
				if (!($eleind === false)) {			
					$text = substr_replace($text, "", $elstart, $eleind-$elstart+strlen($eindel));	
				}
			}			
		} while ( !($elstart === false) && !($eleind === false) );  
	}
	
	function replaceElementTag($startel, $eindel, $replace, &$text)
	{			
		do {		
			$elstart = strpos($text, $startel);
			if (!($elstart === false)) {
				$eleind  = strpos($text, $eindel);
				if (!($eleind === false)) {			
					$text = substr_replace($text, $replace, $elstart, $eleind-$elstart+strlen($eindel));	
				}
			}			
		} while ( !($elstart === false) && !($eleind === false));  
	}
	
	function getLanguageCode()
	{
	  $this->load->model('localisation/language');

		$language_info = $this->model_localisation_language->getLanguage($this->config->get('config_language_id'));

		if ($language_info) {
			$language_code = $language_info['code'];			
		} else {
			$language_code = '';			
		}	
		return $language_code;  
	}		
	
	function contains_array($array){
    foreach($array as $value){
        if(is_array($value)) {
          return true;
        }
    }
    return false;
  }
  
  function remove_arrays(&$array){
    foreach($array as $key => $value){
        if(is_array($value)) {
          unset($array[$key]);
        }
    }
    return $array;
  }
  
  /**
	 * Check language title from config setting
	 *	 
	 * @since   1.7
	 */
  function getLanguageTitle($config, $title, $postfix='') {
    
	  $titles = $this->config->get($config);
    if (!empty($titles[$this->config->get('config_language_id')]['title'])) {
      return html_entity_decode($titles[$this->config->get('config_language_id')]['title'], ENT_QUOTES, 'UTF-8');
    }
    return $title;
  }
  
  /**
	 * Set boolean setting from post or config
	 *	 
	 * @since   1.7
	 */
  function setBooleanFromPostOrConfig(&$data, $config_item, $default = false) {
    if (isset($this->request->post[$config_item])) {
			$data[$config_item] = $this->request->post[$config_item];
		} elseif ($this->request->server['REQUEST_METHOD'] == 'POST') {
			$data[$config_item] = $default;
		} else {
			$data[$config_item] = $this->config->get($config_item);
		}    
	}
	
	/**
	 * Set string setting from post or config
	 *	 
	 * @since   1.7
	 */
  function setStringFromPostOrConfig(&$data, $config_item) {
    if (isset($this->request->post[$config_item])) {
			$data[$config_item] = $this->request->post[$config_item];
		} else {
			$data[$config_item] = $this->config->get($config_item);
		}    
	}
	
	/**
	 * Set language string setting from post or config
	 *	 
	 * @since   1.7
	 */
  function setLanguageStringFromPostOrConfig(&$data, $config_item) {
    if (isset($this->request->post[$config_item])) {
			$data[$config_item] = $this->request->post[$config_item];
		} else {
			$data[$config_item] = $this->config->get($config_item);
		}    
	}
	
	/**
	 * Set array setting from post or config
	 *	 
	 * @since   1.7
	 */
  function setArrayFromPostOrConfig(&$data, $config_item, $default = array()) {
    // Strange issue on PHP 5
    if ($this->config) {
      $item = $this->config->get($config_item);
      if (isset($this->request->post[$config_item])) {
  			$data[$config_item] = $this->request->post[$config_item];
  		} else 
  		if (!empty($item)) {
  			$data[$config_item] = $this->config->get($config_item);
  		} else {
  		  $data[$config_item] = $default;
  		} 
	  }
	}
}

?>
