<?php

class OwcUpdateHelper extends OwcBase {  
  
  const VERSION             = 'v1.1';  
  const VERSION_DATE        = '30-03-2018';  
  
  /**
	 * Method to get the text of an extension
	 *
	 * @return  mixed  Help text on success or false on failure
	 *
	 * @since   1.0
	 */
	public function getText($extension, $page)
	{		  
		if (!isset($this->session->data["OWC_HELPTEXT_".$extension.$page]) || $_SERVER['REMOTE_ADDR'] == '127.0.0.1')
		{
		  $language_code = $this->OwcLoader->OwcHelper->getLanguageCode();  	  
		  
			$url     = 'https://www.openwebcreations.eu/opencart/extensions/index.php?version=' . VERSION. '&language=' . $language_code . '&ext=' . $extension . '&page=' . $page;
			
			$content = $this->getRemoteContent($url);
			if ($content === false || empty($content)) {
			  $content = $this->language->get('error_online_info');
			} 
			
			$this->session->data["OWC_HELPTEXT_".$extension.$page] = $content;			
		}

		return $this->session->data["OWC_HELPTEXT_".$extension.$page];
	}	
	
	/**
	 * Method to get the XML update manifest of the extension
	 *
	 * @return  mixed  SimpleXMLElement on success or false on failure
	 *
	 * @since   1.0
	 */
	public function getUpdateManifest($extension)
	{	  
		$language_code = $this->OwcLoader->OwcHelper->getLanguageCode();  	  
		  
		$url     = 'https://www.openwebcreations.eu/opencart/extensions/index.php?version=' . VERSION. '&language=' . $language_code . '&ext=' . $extension . '&page=manifest';
			
		$content = $this->getRemoteContent($url);

		return $this->getXML($content);		
	}
	
	/**
	 * Method to get remove dat content
	 *
	 * @return  mixed  Url for remote content
	 *
	 * @since   1.0
	 */
	public function getRemoteContent($url)
	{  	  
	  $timeout = 15;

		if (function_exists('curl_version'))
		{
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			$content = curl_exec($ch);

			$curlError = false;

			if (curl_errno($ch))
			{
				$this->logError(curl_error($ch) . ' in ' . __CLASS__ . ' on line ' . __LINE__);				

				$curlError = true;
			}

			curl_close($ch);

			if ($content === false || $curlError === true)
			{
				return false;
			}
		}
		elseif (ini_get('allow_url_fopen'))
		{
			$options = array(
				'http' => array(
					'user_agent'    => 'Firefox',
					'max_redirects' => 1,
					'timeout'       => $timeout,
				)
			);

			$context = stream_context_create($options);
			$content = @file_get_contents($url, null, $context);

			if ($content === false)
			{
				$this->logError('"allow_url_fopen" failed in ' . __CLASS__ . ' on line ' . __LINE__);
				
				return false;
			}
		}
		else
		{
			$this->logError('CURL or "allow_url_fopen" should be enabled in ' . __CLASS__ . ' on line ' . __LINE__);
			return false;
		}

		return $content;		
	}	
	
	/**
	 * Reads a XML file
	 *
	 * @param   string   $data     A well-formed XML string or the path or URL to an XML document if $isUrl is true
	 * @param   string   $options  Optionally used to specify additional Libxml parameters
	 * @param   boolean  $isUrl    True to load a file or url false to load a string
	 *
	 * @return  mixed    SimpleXMLElement on success or false on error
	 *
	 * @since   1.0
	 */
	public function getXML($data, $options = null, $isUrl = false)
	{
		// Disable libxml errors and allow to fetch error information as needed
		libxml_use_internal_errors(true);

		try
		{
			$xml = new SimpleXMLElement($data, $options, $isUrl);
		}
		catch (Exception $e)
		{
			$this->logError($e->getMessage() . ' in ' . __CLASS__ . ' on line1 ' . __LINE__);				

			return false;
		}

		if ($xml === false)
		{
			foreach (libxml_get_errors() as $error)
			{
				$this->logError($error->message . ' in ' . __CLASS__ . ' on line2 ' . __LINE__);				
			}

			libxml_clear_errors();
		}

		return $xml;
	}
}

?>
